from qtpy.QtGui import QIcon, QPixmap

from siriushlacon.utils.consts import CNPEM_IMG as _CNPEM_IMG
from siriushlacon.utils.consts import CNPEM_INVISIBLE_IMG as _CNPEM_INVISIBLE_IMG
from siriushlacon.utils.consts import (
    CNPEM_INVISIBLE_LOGO_IMG as _CNPEM_INVISIBLE_LOGO_IMG,
)
from siriushlacon.utils.consts import LNLS_IMG as _LNLS_IMG
from siriushlacon.utils.consts import LNLS_INVISIBLE_IMG as _LNLS_INVISIBLE_IMG

CNPEM_PIXMAP: QPixmap = QPixmap(_CNPEM_IMG)
CNPEM_ICON: QIcon = QIcon(CNPEM_PIXMAP)

CNPEM_INVISIBLE_PIXMAP: QPixmap = QPixmap(_CNPEM_INVISIBLE_IMG)
CNPEM_INVISIBLE_ICON: QIcon = QIcon(CNPEM_INVISIBLE_PIXMAP)

CNPEM_INVISIBLE_LOGO_PIXMAP: QPixmap = QPixmap(_CNPEM_INVISIBLE_LOGO_IMG)
CNPEM_INVISIBLE_LOGO_ICON: QIcon = QIcon(CNPEM_INVISIBLE_LOGO_PIXMAP)

LNLS_PIXMAP: QPixmap = QPixmap(_LNLS_IMG)
LNLS_ICON: QIcon = QIcon(LNLS_PIXMAP)

LNLS_INVISIBLE_PIXMAP: QPixmap = QPixmap(_LNLS_INVISIBLE_IMG)
LNLS_INVISIBLE_ICON: QIcon = QIcon(LNLS_INVISIBLE_PIXMAP)
